@@ -1,26 +1,35 @@
 import os
+import sys
+import warnings
 import ctypes
 import ctypes.util
 import numpy
 from numpy.ctypeslib import ndpointer
+from galpy.util import galpyWarning
 from galpy.orbit_src.integrateFullOrbit import _parse_pot
 from galpy.util import bovy_coords
 #Find and load the library
-_lib = None
-_libname = ctypes.util.find_library('galpy_actionAngle_c')
-if _libname:
-    _lib = ctypes.CDLL(_libname)
-if _lib is None:
-    import sys
-    for path in sys.path:
-        try:
-            _lib = ctypes.CDLL(os.path.join(path,'galpy_actionAngle_c.so'))
-        except OSError:
-            _lib = None
-        else:
-            break
-if _lib is None:
-    raise IOError('galpy actionAngle_c module not found')
+_lib= None
+outerr= None
+for path in sys.path:
+    try:
+        _lib = ctypes.CDLL(os.path.join(path,'galpy_actionAngle_c.so'))
+    except OSError, e:
+        if os.path.exists(os.path.join(path,'galpy_actionAngle_c.so')): #pragma: no cover
+            outerr= e
+        _lib = None
+    else:
+        break
+if _lib is None: #pragma: no cover
+    if not outerr is None:
+        warnings.warn("actionAngleStaeckel_c extension module not loaded, because of error '%s' " % outerr,
+                      galpyWarning)
+    else:
+        warnings.warn("actionAngleStaeckel_c extension module not loaded, because galpy_actionAngle_c.so image was not found",
+                      galpyWarning)
+    _ext_loaded= False
+else:
+    _ext_loaded= True
 
 def actionAngleStaeckel_c(pot,delta,R,vR,vT,z,vz,u0=None):
     """
@@ -382,8 +391,9 @@ def actionAngleFreqAngleStaeckel_c(pot,delta,R,vR,vT,z,vz,phi,u0=None):
     if f_cont[3]: z= numpy.asfortranarray(z)
     if f_cont[4]: vz= numpy.asfortranarray(vz)
     if f_cont[5]: u0= numpy.asfortranarray(u0)
-
-    Anglephi= (Anglephi + phi % (2.*numpy.pi)) % (2.*numpy.pi)
+    
+    badAngle = Anglephi != 9999.99
+    Anglephi[badAngle]= (Anglephi[badAngle] + phi[badAngle] % (2.*numpy.pi)) % (2.*numpy.pi)
     Anglephi[Anglephi < 0.]+= 2.*numpy.pi
 
     return (jr,jz,Omegar,Omegaphi,Omegaz,Angler,
