@@ -1,9 +1,14 @@
 #A 'Binney' quasi-isothermal DF
 import math
+import warnings
 import numpy
 from scipy import optimize, interpolate, integrate
 from galpy import potential
 from galpy import actionAngle
+from galpy.actionAngle import actionAngleIsochrone
+from galpy.potential import IsochronePotential
+from galpy.orbit import Orbit
+from galpy.util import galpyWarning
 _NSIGMA=4
 _DEFAULTNGL=10
 _DEFAULTNGL2=20
@@ -78,7 +83,12 @@ class quasiisothermaldf:
             raise IOError("aA= must be set")
         self._aA= aA
         if not self._aA._pot == self._pot:
-            raise IOError("Potential in aA does not appear to be the same as given potential pot")
+            if not isinstance(self._aA,actionAngleIsochrone):
+                raise IOError("Potential in aA does not appear to be the same as given potential pot")
+            elif isinstance(self._pot,IsochronePotential) and \
+                    not self._aA.b == self._pot.b and \
+                    not self._aA.amp == self._pot._amp:
+                raise IOError("Potential in aA does not appear to be the same as given potential pot")
         self._cutcounter= cutcounter
         if _precomputerg:
             if _precomputergrmax is None:
@@ -98,6 +108,10 @@ class quasiisothermaldf:
             self._rls= None
             self._precomputergnr= None
             self._precomputergLzgrid= None
+            self._precomputergLzmin= \
+                numpy.finfo(numpy.dtype(numpy.float64)).max
+            self._precomputergLzmax= \
+                numpy.finfo(numpy.dtype(numpy.float64)).min
         self._precomputerg= _precomputerg
         self._glxdef, self._glwdef= \
             numpy.polynomial.legendre.leggauss(_DEFAULTNGL)
@@ -177,7 +191,7 @@ class quasiisothermaldf:
             nu= None
             Omega= None
         #First parse args
-        if len(args) == 1: #(jr,lz,jz)
+        if len(args) == 1 and not isinstance(args[0],Orbit): #(jr,lz,jz)
             jr,lz,jz= args[0]
         else:
             #Use self._aA to calculate the actions
@@ -580,7 +594,8 @@ class quasiisothermaldf:
                             vrs,vts,vzs)
             else:
                 return numpy.mean(Is)*sigmaR1**(2.+n+m)*gamma**(1.+m)*sigmaz1**(1.+o)
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
+            warnings.warn("Calculations using direct numerical integration using tplquad is not recommended and extremely slow; it has also not been carefully tested",galpyWarning)
             return integrate.tplquad(_vmomentsurfaceIntegrand,
                                      1./gamma*(thisvc-va)/sigmaR1-nsigma,
                                      1./gamma*(thisvc-va)/sigmaR1+nsigma,
@@ -645,13 +660,14 @@ class quasiisothermaldf:
                 vzs= numpy.random.normal(size=nmc)
             else:
                 vzs= _vzs
-            Is= _jmomentsurfaceMCIntegrand(vzs,vrs,vts,nump.ones(nmc)*R,numpy.ones(nmc)*z,self,sigmaR1,gamma,sigmaz1,mvT,n,m,o)
+            Is= _jmomentsurfaceMCIntegrand(vzs,vrs,vts,numpy.ones(nmc)*R,numpy.ones(nmc)*z,self,sigmaR1,gamma,sigmaz1,mvT,n,m,o)
             if _returnmc:
                 return (numpy.mean(Is)*sigmaR1**2.*gamma*sigmaz1,
                         vrs,vts,vzs)
             else:
                 return numpy.mean(Is)*sigmaR1**2.*gamma*sigmaz1
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
+            warnings.warn("Calculations using direct numerical integration using tplquad is not recommended and extremely slow; it has also not been carefully tested",galpyWarning)
             return integrate.tplquad(_jmomentsurfaceIntegrand,
                                      1./gamma*(thisvc-va)/sigmaR1-nsigma,
                                      1./gamma*(thisvc-va)/sigmaR1+nsigma,
@@ -743,7 +759,7 @@ class quasiisothermaldf:
                                            ngl=ngl,gl=gl,
                                            _glqeval=glqeval,
                                            **kwargs)/surfmass
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             return (self.vmomentdensity(R,z,2.,0.,0.,
                                            nsigma=nsigma,mc=mc,nmc=nmc,
                                            **kwargs)/
@@ -800,7 +816,7 @@ class quasiisothermaldf:
                                            ngl=ngl,gl=gl,
                                            _glqeval=glqeval,
                                            **kwargs)/surfmass
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             return (self.vmomentdensity(R,z,1.,0.,1.,
                                            nsigma=nsigma,mc=mc,nmc=nmc,
                                            **kwargs)/
@@ -926,7 +942,7 @@ class quasiisothermaldf:
                                            ngl=ngl,gl=gl,
                                            _glqeval=glqeval,
                                            **kwargs)/surfmass
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             return (self.vmomentdensity(R,z,0.,0.,2.,
                                            nsigma=nsigma,mc=mc,nmc=nmc,
                                            **kwargs)/
@@ -987,7 +1003,7 @@ class quasiisothermaldf:
                                            ngl=ngl,gl=gl,
                                            _glqeval=glqeval,
                                            **kwargs)/surfmass
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             return (self.vmomentdensity(R,z,0.,1.,0.,
                                            nsigma=nsigma,mc=mc,nmc=nmc,
                                            **kwargs)/
@@ -1044,7 +1060,7 @@ class quasiisothermaldf:
                                            ngl=ngl,gl=gl,
                                            _glqeval=glqeval,
                                            **kwargs)/surfmass
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             return (self.vmomentdensity(R,z,1.,0.,0.,
                                            nsigma=nsigma,mc=mc,nmc=nmc,
                                            **kwargs)/
@@ -1101,7 +1117,7 @@ class quasiisothermaldf:
                                            ngl=ngl,gl=gl,
                                            _glqeval=glqeval,
                                            **kwargs)/surfmass
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             return (self.vmomentdensity(R,z,0.,0.,1.,
                                            nsigma=nsigma,mc=mc,nmc=nmc,
                                            **kwargs)/
@@ -1168,7 +1184,7 @@ class quasiisothermaldf:
                                            _glqeval=glqeval,
                                            **kwargs)/surfmass-mvt**2.
 
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             surfmass= self.vmomentdensity(R,z,0.,0.,0.,
                                               nsigma=nsigma,mc=mc,nmc=nmc,
                                               **kwargs)
@@ -1214,7 +1230,7 @@ class quasiisothermaldf:
                                            nsigma=nsigma,mc=mc,nmc=nmc,_returnmc=False,
                                            _vrs=vrs,_vts=vts,_vzs=vzs,
                                                              **kwargs)/surfmass
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             return (self.jmomentdensity(R,z,1.,0.,0.,
                                            nsigma=nsigma,mc=mc,nmc=nmc,
                                            **kwargs)/
@@ -1257,7 +1273,7 @@ class quasiisothermaldf:
                                            nsigma=nsigma,mc=mc,nmc=nmc,_returnmc=False,
                                            _vrs=vrs,_vts=vts,_vzs=vzs,
                                                              **kwargs)/surfmass
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             return (self.jmomentdensity(R,z,0.,1.,0.,
                                            nsigma=nsigma,mc=mc,nmc=nmc,
                                            **kwargs)/
@@ -1300,7 +1316,7 @@ class quasiisothermaldf:
                                            nsigma=nsigma,mc=mc,nmc=nmc,_returnmc=False,
                                            _vrs=vrs,_vts=vts,_vzs=vzs,
                                                              **kwargs)/surfmass
-        else:
+        else: #pragma: no cover because this is too slow; a warning is shown
             return (self.jmomentdensity(R,z,0.,0.,1.,
                                            nsigma=nsigma,mc=mc,nmc=nmc,
                                            **kwargs)/
@@ -1424,7 +1440,7 @@ class quasiisothermaldf:
                                          vzgl.flatten(),
                                          log=True),
                                     (ngl,ngl))
-            return numpy.sum(numpy.exp(logqeval)*vTglw*vzglw*sigmaz1)
+            return numpy.sum(numpy.exp(logqeval)*vTglw*vzglw*sigmaz1)*1.5
 
     def pvT(self,vT,R,z,gl=True,ngl=_DEFAULTNGL2):
         """
@@ -1583,9 +1599,10 @@ class quasiisothermaldf:
                 vz= vz+numpy.zeros(ngl*ngl)
                 nR= 1
                 scalarOut= True
+                vRgl= vRgl.flatten()
             vRgl*= numpy.tile(4.*sigmaR1/2.,(ngl,ngl,1)).T.flatten()
             #evaluate
-            if _jr is None:
+            if _jr is None and _rg is None:
                 logqeval, jr, lz, jz, rg, kappa, nu, Omega= self(R,
                                                                  vRgl.flatten(),
                                                                  vTgl.flatten(),
@@ -1897,17 +1914,9 @@ class quasiisothermaldf:
         else:
             if lz > self._precomputergLzmax or lz < self._precomputergLzmin:
                 return potential.rl(self._pot,lz)
-            return self._rgInterp(lz)
+            return numpy.atleast_1d(self._rgInterp(lz))
 
-def _surfaceIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1):
-    """Internal function that is the integrand for the surface mass integration"""
-    return df(R,vR*sigmaR1,vT*sigmaR1*gamma,z,vz*sigmaz1)
-
-def _surfaceMCIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1,mvT):
-    """Internal function that is the integrand for the surface mass integration"""
-    return df(R,vR*sigmaR1,vT*sigmaR1*gamma,z,vz*sigmaz1)*numpy.exp(vR**2./2.+(vT-mvT)**2./2.+vz**2./2.)
-
-def _vmomentsurfaceIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1,n,m,o):
+def _vmomentsurfaceIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1,n,m,o): #pragma: no cover because this is too slow; a warning is shown
     """Internal function that is the integrand for the vmomentsurface mass integration"""
     return vR**n*vT**m*vz**o*df(R,vR*sigmaR1,vT*sigmaR1*gamma,z,vz*sigmaz1)
 
@@ -1915,7 +1924,7 @@ def _vmomentsurfaceMCIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1,mvT,n,m,o):
     """Internal function that is the integrand for the vmomentsurface mass integration"""
     return vR**n*vT**m*vz**o*df(R,vR*sigmaR1,vT*sigmaR1*gamma,z,vz*sigmaz1)*numpy.exp(vR**2./2.+(vT-mvT)**2./2.+vz**2./2.)
 
-def _jmomentsurfaceIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1,n,m,o):
+def _jmomentsurfaceIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1,n,m,o): #pragma: no cover because this is too slow; a warning is shown
     """Internal function that is the integrand for the vmomentsurface mass integration"""
     return df(R,vR*sigmaR1,vT*sigmaR1*gamma,z,vz*sigmaz1,
               func= (lambda x,y,z: x**n*y**m*z**o))
@@ -1925,26 +1934,3 @@ def _jmomentsurfaceMCIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1,mvT,n,m,o):
     return df(R,vR*sigmaR1,vT*sigmaR1*gamma,z,vz*sigmaz1,
               func=(lambda x,y,z: x**n*y**m*z**o))\
               *numpy.exp(vR**2./2.+(vT-mvT)**2./2.+vz**2./2.)
-
-def _sigmaR2surfaceIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1):
-    """Internal function that is the integrand for the sigma-squared times
-    surface mass integration"""
-    return (vR*sigmaR1)**2.*df(R,vR*sigmaR1,vT*sigmaR1*gamma,z,vz*sigmaz1)
-
-def _sigmaz2surfaceIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1):
-    """Internal function that is the integrand for the sigma-squared times
-    surface mass integration"""
-    return (vz*sigmaz1)**2.*df(R,vR*sigmaR1,vT*sigmaR1*gamma,z,vz*sigmaz1)
-
-def _meanvphisurfaceIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1):
-    """Internal function that is the integrand for the <vT> times
-    surface mass integration"""
-    return (vT*sigmaR1*gamma)*df(R,vR*sigmaR1,vT*sigmaR1*gamma,
-                                     z,vz*sigmaz1)
-
-def _meanvphi2surfaceIntegrand(vz,vR,vT,R,z,df,sigmaR1,gamma,sigmaz1):
-    """Internal function that is the integrand for the sigma-squared times
-    surface mass integration"""
-    return (vT*sigmaR1*gamma)**2.*df(R,vR*sigmaR1,vT*sigmaR1*gamma,
-                                     z,vz*sigmaz1)
-
