@@ -6,29 +6,50 @@ from galpy.potential_src.Potential import evaluateRforces, evaluatezforces,\
 import galpy.util.bovy_plot as plot
 import galpy.util.bovy_symplecticode as symplecticode
 from galpy.orbit_src.FullOrbit import _integrateFullOrbit
+from galpy.util.bovy_conversion import physical_conversion
 from OrbitTop import OrbitTop
 class RZOrbit(OrbitTop):
     """Class that holds and integrates orbits in axisymetric potentials 
     in the (R,z) plane"""
-    def __init__(self,vxvv=[1.,0.,0.9,0.,0.1]):
+    def __init__(self,vxvv=[1.,0.,0.9,0.,0.1],vo=220.,ro=8.0,zo=0.025,
+                 solarmotion=nu.array([-10.1,4.0,6.7])):
         """
         NAME:
+
            __init__
+
         PURPOSE:
+
            intialize an RZ-orbit
+
         INPUT:
+
            vxvv - initial condition [R,vR,vT,z,vz]
+
+           vo - circular velocity at ro (km/s)
+
+           ro - distance from vantage point to GC (kpc)
+
+           zo - offset toward the NGP of the Sun wrt the plane (kpc)
+
+           solarmotion - value in [-U,V,W] (km/s)
+
         OUTPUT:
+
            (none)
+
         HISTORY:
+
            2010-07-10 - Written - Bovy (NYU)
+
+           2014-06-11 - Added conversion kwargs to physical coordinates - Bovy (IAS)
+
         """
-        self.vxvv= vxvv
-        #For boundary-condition integration
-        self._BCIntegrateFunction= _integrateRZOrbit
+        OrbitTop.__init__(self,vxvv=vxvv,
+                          ro=ro,zo=zo,vo=vo,solarmotion=solarmotion)
         return None
 
-    def integrate(self,t,pot,method='odeint'):
+    def integrate(self,t,pot,method='symplec4_c'):
         """
         NAME:
            integrate
@@ -37,8 +58,12 @@ class RZOrbit(OrbitTop):
         INPUT:
            t - list of times at which to output (0 has to be in this!)
            pot - potential instance or list of instances
-           method= 'odeint' for scipy's odeint integrator, 'leapfrog' for
-                   a simple symplectic integrator
+           method= 'odeint' for scipy's odeint
+                   'leapfrog' for a simple leapfrog implementation
+                   'leapfrog_c' for a simple leapfrog implementation in C
+                   'rk4_c' for a 4th-order Runge-Kutta integrator in C
+                   'rk6_c' for a 6-th order Runge-Kutta integrator in C
+                   'dopr54_c' for a Dormand-Prince integrator in C (generally the fastest)
         OUTPUT:
            (none) (get the actual orbit using getOrbit()
         HISTORY:
@@ -50,6 +75,7 @@ class RZOrbit(OrbitTop):
         self._pot= pot
         self.orbit= _integrateRZOrbit(self.vxvv,pot,t,method)
 
+    @physical_conversion('energy')
     def E(self,*args,**kwargs):
         """
         NAME:
@@ -95,6 +121,7 @@ class RZOrbit(OrbitTop):
                                  +thiso[2,ii]**2./2.\
                                  +thiso[4,ii]**2./2. for ii in range(len(t))])
 
+    @physical_conversion('energy')
     def ER(self,*args,**kwargs):
         """
         NAME:
@@ -138,6 +165,7 @@ class RZOrbit(OrbitTop):
                                  +thiso[1,ii]**2./2.\
                                  +thiso[2,ii]**2./2. for ii in range(len(t))])
 
+    @physical_conversion('energy')
     def Ez(self,*args,**kwargs):
         """
         NAME:
@@ -184,6 +212,7 @@ class RZOrbit(OrbitTop):
                                                 t=t[ii])\
                                  +thiso[4,ii]**2./2. for ii in range(len(t))])
 
+    @physical_conversion('energy')
     def Jacobi(self,*args,**kwargs):
         """
         NAME:
@@ -192,7 +221,7 @@ class RZOrbit(OrbitTop):
            calculate the Jacobi integral of the motion
         INPUT:
            t - (optional) time at which to get the radius
-           OmegaP= pattern speed of rotating frame
+           OmegaP= pattern speed of rotating frame (scalar)
            pot= potential instance or list of such instances
         OUTPUT:
            Jacobi integral
@@ -221,13 +250,16 @@ class RZOrbit(OrbitTop):
         else:
             OmegaP= kwargs['OmegaP']
             kwargs.pop('OmegaP')
-        if not isinstance(OmegaP,(int,float)) and len(OmegaP) == 3:
-            if isinstance(OmegaP,list): thisOmegaP= nu.array(OmegaP)
-            else: thisOmegaP= OmegaP
-            return self.E(*args,**kwargs)-nu.dot(thisOmegaP,
-                                                 self.L(*args,**kwargs))
+        #Make sure you are not using physical coordinates
+        old_physical= kwargs.get('use_physical',None)
+        kwargs['use_physical']= False
+        thiso= self(*args,**kwargs)
+        out= self.E(*args,**kwargs)-OmegaP*thiso[0]*thiso[2]
+        if not old_physical is None:
+            kwargs['use_physical']= old_physical
         else:
-            return self.E(*args,**kwargs)-OmegaP*self.L(*args,**kwargs)[:,2]
+            kwargs.pop('use_physical')
+        return out
 
     def e(self,analytic=False,pot=None):
         """
@@ -253,7 +285,8 @@ class RZOrbit(OrbitTop):
             self.rs= nu.sqrt(self.orbit[:,0]**2.+self.orbit[:,3]**2.)
         return (nu.amax(self.rs)-nu.amin(self.rs))/(nu.amax(self.rs)+nu.amin(self.rs))
 
-    def rap(self,analytic=False,pot=None):
+    @physical_conversion('position')
+    def rap(self,analytic=False,pot=None,**kwargs):
         """
         NAME:
            rap
@@ -277,7 +310,8 @@ class RZOrbit(OrbitTop):
             self.rs= nu.sqrt(self.orbit[:,0]**2.+self.orbit[:,3]**2.)
         return nu.amax(self.rs)
 
-    def rperi(self,analytic=False,pot=None):
+    @physical_conversion('position')
+    def rperi(self,analytic=False,pot=None,**kwargs):
         """
         NAME:
            rperi
@@ -301,7 +335,8 @@ class RZOrbit(OrbitTop):
             self.rs= nu.sqrt(self.orbit[:,0]**2.+self.orbit[:,3]**2.)
         return nu.amin(self.rs)
 
-    def zmax(self,analytic=False,pot=None):
+    @physical_conversion('position')
+    def zmax(self,analytic=False,pot=None,**kwargs):
         """
         NAME:
            zmax
@@ -321,175 +356,46 @@ class RZOrbit(OrbitTop):
             raise AttributeError("Integrate the orbit first")
         return nu.amax(nu.fabs(self.orbit[:,3]))
 
-    def plotE(self,*args,**kwargs):
-        """
-        NAME:
-           plotE
-        PURPOSE:
-           plot E(.) along the orbit
-        INPUT:
-           pot= - Potential instance or list of instances in which the orbit was
-                 integrated
-           d1= - plot E vs d1: e.g., 't', 'z', 'R', 'vR', 'vT', 'vz'      
-           +bovy_plot.bovy_plot inputs
-        OUTPUT:
-           figure to output device
-        HISTORY:
-           2010-07-10 - Written - Bovy (NYU)
-        """
-        labeldict= {'t':r'$t$','R':r'$R$','vR':r'$v_R$','vT':r'$v_T$',
-                    'z':r'$z$','vz':r'$v_z$','phi':r'$\phi$',
-                    'x':r'$x$','y':r'$y$','vx':r'$v_x$','vy':r'$v_y$'}
-        if not kwargs.has_key('pot'):
-            try:
-                pot= self._pot
-            except AttributeError:
-                raise AttributeError("Integrate orbit first or specify pot=")
-        else:
-            pot= kwargs['pot']
-            kwargs.pop('pot')
-        if kwargs.has_key('d1'):
-            d1= kwargs['d1']
-            kwargs.pop('d1')
-        else:
-            d1= 't'
-        self.Es= [evaluatePotentials(self.orbit[ii,0],self.orbit[ii,3],pot,
-                                     t=self.t[ii])+
-                  self.orbit[ii,1]**2./2.+self.orbit[ii,2]**2./2.+
-                  self.orbit[ii,4]**2./2. for ii in range(len(self.t))]
-        if not kwargs.has_key('xlabel'):
-            kwargs['xlabel']= labeldict[d1]
-        if not kwargs.has_key('ylabel'):
-            kwargs['ylabel']= r'$E$'
-        if d1 == 't':
-            plot.bovy_plot(nu.array(self.t),nu.array(self.Es)/self.Es[0],
-                           *args,**kwargs)
-        elif d1 == 'z':
-            plot.bovy_plot(self.orbit[:,3],nu.array(self.Es)/self.Es[0],
-                           *args,**kwargs)
-        elif d1 == 'R':
-            plot.bovy_plot(self.orbit[:,0],nu.array(self.Es)/self.Es[0],
-                           *args,**kwargs)
-        elif d1 == 'vR':
-            plot.bovy_plot(self.orbit[:,1],nu.array(self.Es)/self.Es[0],
-                           *args,**kwargs)
-        elif d1 == 'vT':
-            plot.bovy_plot(self.orbit[:,2],nu.array(self.Es)/self.Es[0],
-                           *args,**kwargs)
-        elif d1 == 'vz':
-            plot.bovy_plot(self.orbit[:,4],nu.array(self.Es)/self.Es[0],
-                           *args,**kwargs)
-
     def plotEz(self,*args,**kwargs):
         """
         NAME:
            plotEz
         PURPOSE:
-           plot E_z(.) along the orbit
+           plot Ez(.) along the orbit
         INPUT:
-           pot= Potential instance or list of instances in which the orbit was
-                 integrated
-           d1= - plot Ez vs d1: e.g., 't', 'z', 'R', 'vR', 'vT', 'vz'      
-           +bovy_plot.bovy_plot inputs
+           bovy_plot.bovy_plot inputs
         OUTPUT:
            figure to output device
         HISTORY:
-           2010-07-10 - Written - Bovy (NYU)
+           2014-06-16 - Written - Bovy (IAS)
         """
-        labeldict= {'t':r'$t$','R':r'$R$','vR':r'$v_R$','vT':r'$v_T$',
-                    'z':r'$z$','vz':r'$v_z$','phi':r'$\phi$',
-                    'x':r'$x$','y':r'$y$','vx':r'$v_x$','vy':r'$v_y$'}
-        if not kwargs.has_key('pot'):
-            try:
-                pot= self._pot
-            except AttributeError:
-                raise AttributeError("Integrate orbit first or specify pot=")
+        if kwargs.get('normed',False):
+            kwargs['d2']= 'Eznorm'
         else:
-            pot= kwargs['pot']
-            kwargs.pop('pot')
-        if kwargs.has_key('d1'):
-            d1= kwargs['d1']
-            kwargs.pop('d1')
-        else:
-            d1= 't'
-        self.Ezs= [evaluatePotentials(self.orbit[ii,0],self.orbit[ii,3],pot,
-                                      t=self.t[ii])-
-                   evaluatePotentials(self.orbit[ii,0],0.,pot,t=self.t[ii])+
-                   self.orbit[ii,4]**2./2. for ii in range(len(self.t))]
-        if not kwargs.has_key('xlabel'):
-            kwargs['xlabel']= labeldict[d1]
-        if not kwargs.has_key('ylabel'):
-            kwargs['ylabel']= r'$E_z$'
-        if d1 == 't':
-            plot.bovy_plot(nu.array(self.t),nu.array(self.Ezs)/self.Ezs[0],
-                           *args,**kwargs)
-        elif d1 == 'z':
-            plot.bovy_plot(self.orbit[:,3],nu.array(self.Ezs)/self.Ezs[0],
-                           *args,**kwargs)
-        elif d1 == 'R':
-            plot.bovy_plot(self.orbit[:,0],nu.array(self.Ezs)/self.Ezs[0],
-                           *args,**kwargs)
-        elif d1 == 'vR':
-            plot.bovy_plot(self.orbit[:,1],nu.array(self.Ezs)/self.Ezs[0],
-                           *args,**kwargs)
-        elif d1 == 'vT':
-            plot.bovy_plot(self.orbit[:,2],nu.array(self.Ezs)/self.Ezs[0],
-                           *args,**kwargs)
-        elif d1 == 'vz':
-            plot.bovy_plot(self.orbit[:,4],nu.array(self.Ezs)/self.Ezs[0],
-                           *args,**kwargs)
-         
-    def plotJacobi(self,*args,**kwargs):
+            kwargs['d2']= 'Ez'
+        if kwargs.has_key('normed'): kwargs.pop('normed')
+        self.plot(*args,**kwargs)
+        
+    def plotER(self,*args,**kwargs):
         """
         NAME:
-           plotJacobi
+           plotER
         PURPOSE:
-           plot the Jacobi integral(.) along the orbit
+           plot ER(.) along the orbit
         INPUT:
-           OmegaP= pattern speed
-           pot= Potential instance or list of instances in which the orbit was
-                 integrated
-           d1= - plot Jacobi vs d1: e.g., 't', 'z', 'R', 'vR', 'vT', 'vz'      
-           +bovy_plot.bovy_plot inputs
+           bovy_plot.bovy_plot inputs
         OUTPUT:
            figure to output device
         HISTORY:
-           2011-10-10 - Written - Bovy (IAS)
+           2014-06-16 - Written - Bovy (IAS)
         """
-        labeldict= {'t':r'$t$','R':r'$R$','vR':r'$v_R$','vT':r'$v_T$',
-                    'z':r'$z$','vz':r'$v_z$','phi':r'$\phi$',
-                    'x':r'$x$','y':r'$y$','vx':r'$v_x$','vy':r'$v_y$'}
-        Js= self.Jacobi(self.t,**kwargs)
-        if kwargs.has_key('OmegaP'): kwargs.pop('OmegaP')
-        if kwargs.has_key('pot'): kwargs.pop('pot')
-        if kwargs.has_key('d1'):
-            d1= kwargs['d1']
-            kwargs.pop('d1')
+        if kwargs.get('normed',False):
+            kwargs['d2']= 'ERnorm'
         else:
-            d1= 't'
-        if not kwargs.has_key('xlabel'):
-            kwargs['xlabel']= labeldict[d1]
-        if not kwargs.has_key('ylabel'):
-            kwargs['ylabel']= r'$E$'
-        if d1 == 't':
-            plot.bovy_plot(nu.array(self.t),Js/Js[0],
-                           *args,**kwargs)
-        elif d1 == 'z':
-            plot.bovy_plot(self.orbit[:,3],Js/Js[0],
-                           *args,**kwargs)
-        elif d1 == 'R':
-            plot.bovy_plot(self.orbit[:,0],Js/Js[0],
-                           *args,**kwargs)
-        elif d1 == 'vR':
-            plot.bovy_plot(self.orbit[:,1],Js/Js[0],
-                           *args,**kwargs)
-        elif d1 == 'vT':
-            plot.bovy_plot(self.orbit[:,2],Js/Js[0],
-                           *args,**kwargs)
-        elif d1 == 'vz':
-            plot.bovy_plot(self.orbit[:,4],Js/Js[0],
-                           *args,**kwargs)
-
+            kwargs['d2']= 'ER'
+        if kwargs.has_key('normed'): kwargs.pop('normed')
+        self.plot(*args,**kwargs)
+        
     def plotEzJz(self,*args,**kwargs):
         """
         NAME:
@@ -552,9 +458,6 @@ class RZOrbit(OrbitTop):
             plot.bovy_plot(self.orbit[:,4],nu.array(self.EzJz)/self.EzJz[0],
                            *args,**kwargs)
 
-    def _callRect(self,*args):
-        raise AttributeError("Cannot transform RZ-only orbit to rectangular coordinates")
-
 def _integrateRZOrbit(vxvv,pot,t,method):
     """
     NAME:
@@ -572,6 +475,16 @@ def _integrateRZOrbit(vxvv,pot,t,method):
     HISTORY:
        2010-04-16 - Written - Bovy (NYU)
     """
+    #First check that the potential has C
+    if '_c' in method:
+        if isinstance(pot,list):
+            allHasC= nu.prod([p.hasC for p in pot])
+        else:
+            allHasC= pot.hasC
+        if not allHasC and ('leapfrog' in method or 'symplec' in method):
+            method= 'leapfrog'
+        elif not allHasC:
+            method= 'odeint'
     if method.lower() == 'leapfrog' \
             or method.lower() == 'leapfrog_c' or method.lower() == 'rk4_c' \
             or method.lower() == 'rk6_c' or method.lower() == 'symplec4_c' \
@@ -620,13 +533,4 @@ def _RZEOM(y,t,pot,l2):
             l2/y[0]**3.+evaluateRforces(y[0],y[2],pot,t=t),
             y[3],
             evaluatezforces(y[0],y[2],pot,t=t)]
-"""
-def _integrateBCFuncRZ(t,vxvv,pot,method,bc,to):
-    if t == to: return bc(vxvv)
-    #Determine number of ts
-    nts= int(nu.ceil(t-to))+1 #very simple estimate
-    tin= nu.linspace(to,t,nts)
-    orb= _integrateRZOrbit(vxvv,pot,tin,method)
-    return bc(orb[nts-1,:])
-"""
 
