@@ -1,9 +1,11 @@
+import os
+import pickle
 import numpy as nu
+from scipy import integrate
 import galpy.util.bovy_plot as plot
 from Potential import PotentialError, Potential, lindbladR
 from plotRotcurve import plotRotcurve
-from plotEscapecurve import plotEscapecurve
-_INF= 1000000.
+from plotEscapecurve import plotEscapecurve, _INF
 class planarPotential:
     """Class representing 2D (R,\phi) potentials"""
     def __init__(self,amp=1.):
@@ -12,6 +14,7 @@ class planarPotential:
         self.isNonAxi= True #Gets reset by planarAxiPotential
         self.isRZ= False
         self.hasC= False
+        self.hasC_dxdv= False
         return None
 
     def __call__(self,R,phi=0.,t=0.,dR=0,dphi=0):
@@ -43,10 +46,21 @@ class planarPotential:
            2010-07-13 - Written - Bovy (NYU)
 
         """
-        try:
-            return self._amp*self._evaluate(R,phi=phi,t=t,dR=dR,dphi=dphi)
-        except AttributeError:
-            raise PotentialError("'_evaluate' function not implemented for this potential")
+        if dR == 0 and dphi == 0:
+            try:
+                return self._amp*self._evaluate(R,phi=phi,t=t)
+            except AttributeError: #pragma: no cover
+                raise PotentialError("'_evaluate' function not implemented for this potential")
+        elif dR == 1 and dphi == 0:
+            return -self.Rforce(R,phi=phi,t=t)
+        elif dR == 0 and dphi == 1:
+            return -self.phiforce(R,phi=phi,t=t)
+        elif dR == 2 and dphi == 0:
+            return self.R2deriv(R,phi=phi,t=t)
+        elif dR == 0 and dphi == 2:
+            return self.phi2deriv(R,phi=phi,t=t)
+        elif dR == 1 and dphi == 1:
+            return self.Rphideriv(R,phi=phi,t=t)
 
     def Rforce(self,R,phi=0.,t=0.):
         """
@@ -77,7 +91,7 @@ class planarPotential:
         """
         try:
             return self._amp*self._Rforce(R,phi=phi,t=t)
-        except AttributeError:
+        except AttributeError: #pragma: no cover
             raise PotentialError("'_Rforce' function not implemented for this potential")
 
     def phiforce(self,R,phi=0.,t=0.):
@@ -109,7 +123,7 @@ class planarPotential:
         """
         try:
             return self._amp*self._phiforce(R,phi=phi,t=t)
-        except AttributeError:
+        except AttributeError: #pragma: no cover
             raise PotentialError("'_phiforce' function not implemented for this potential")
 
     def R2deriv(self,R,phi=0.,t=0.):
@@ -141,9 +155,73 @@ class planarPotential:
         """
         try:
             return self._amp*self._R2deriv(R,phi=phi,t=t)
-        except AttributeError:
+        except AttributeError: #pragma: no cover
             raise PotentialError("'_R2deriv' function not implemented for this potential")      
 
+    def phi2deriv(self,R,phi=0.,t=0.):
+        """
+        NAME:
+
+           phi2deriv
+
+        PURPOSE:
+
+           evaluate the second azimuthal derivative
+
+        INPUT:
+
+           R - Cylindrical radius
+
+           phi= azimuth (optional)
+
+           t= time (optional)
+
+        OUTPUT:
+
+           d2phi/daz2
+
+        HISTORY:
+
+           2014-04-06 - Written - Bovy (IAS)
+
+        """
+        try:
+            return self._amp*self._phi2deriv(R,phi=phi,t=t)
+        except AttributeError: #pragma: no cover
+            raise PotentialError("'_phi2deriv' function not implemented for this potential")      
+
+    def Rphideriv(self,R,phi=0.,t=0.):
+        """
+        NAME:
+
+           Rphideriv
+
+        PURPOSE:
+
+           evaluate the mixed radial and azimuthal  derivative
+
+        INPUT:
+
+           R - Cylindrical radius
+
+           phi= azimuth (optional)
+
+           t= time (optional)
+
+        OUTPUT:
+
+           d2phi/dR d az
+
+        HISTORY:
+
+           2014-05-21 - Written - Bovy (IAS)
+
+        """
+        try:
+            return self._amp*self._Rphideriv(R,phi=phi,t=t)
+        except AttributeError: #pragma: no cover
+            raise PotentialError("'_Rphideriv' function not implemented for this potential")      
+
     def plot(self,*args,**kwargs):
         """
         NAME:
@@ -160,7 +238,7 @@ class planarPotential:
         HISTORY:
            2010-07-13 - Written - Bovy (NYU)
         """
-        plotplanarPotentials(self,*args,**kwargs)
+        return plotplanarPotentials(self,*args,**kwargs)
 
 class planarAxiPotential(planarPotential):
     """Class representing axisymmetric planar potentials"""
@@ -172,7 +250,7 @@ class planarAxiPotential(planarPotential):
     def _phiforce(self,R,phi=0.,t=0.):
         return 0.
 
-    def _phi2deriv(self,R,z,phi=0.,t=0.):
+    def _phi2deriv(self,R,phi=0.,t=0.): #pragma: no cover
         """
         NAME:
            _phi2deriv
@@ -190,7 +268,7 @@ class planarAxiPotential(planarPotential):
         """
         return 0.
 
-    def _Rphideriv(self,R,z,phi=0.,t=0.):
+    def _Rphideriv(self,R,phi=0.,t=0.): #pragma: no cover
         """
         NAME:
            _Rphideriv
@@ -377,7 +455,7 @@ class planarAxiPotential(planarPotential):
            2010-07-13 - Written - Bovy (NYU)
 
         """
-        plotRotcurve(self,*args,**kwargs)
+        return plotRotcurve(self,*args,**kwargs)
 
     def plotEscapecurve(self,*args,**kwargs):
         """
@@ -408,7 +486,7 @@ class planarAxiPotential(planarPotential):
            2010-07-13 - Written - Bovy (NYU)
 
         """
-        plotEscapecurve(self,*args,**kwargs)
+        return plotEscapecurve(self,*args,**kwargs)
 
 class planarPotentialFromRZPotential(planarAxiPotential):
     """Class that represents an axisymmetic planar potential derived from a 
@@ -428,9 +506,11 @@ class planarPotentialFromRZPotential(planarAxiPotential):
         """
         planarAxiPotential.__init__(self,amp=1.)
         self._RZPot= RZPot
+        self.hasC= RZPot.hasC
+        self.hasC_dxdv= RZPot.hasC_dxdv
         return None
 
-    def _evaluate(self,R,phi=0.,t=0.,dR=0,dphi=0):
+    def _evaluate(self,R,phi=0.,t=0.):
         """
         NAME:
            _evaluate
@@ -445,7 +525,7 @@ class planarPotentialFromRZPotential(planarAxiPotential):
         HISTORY:
            2010-07-13 - Written - Bovy (NYU)
         """
-        return self._RZPot(R,0.,t=t,dR=dR,dphi=dphi)
+        return self._RZPot(R,0.,t=t)
             
     def _Rforce(self,R,phi=0.,t=0.):
         """
@@ -558,7 +638,8 @@ def evaluateplanarPotentials(R,Pot,phi=None,t=0.,dR=0,dphi=0):
         nonAxi= Pot.isNonAxi
     if nonAxi and phi is None:
         raise PotentialError("The (list of) planarPotential instances is non-axisymmetric, but you did not provide phi")
-    if isinstance(Pot,list):
+    if isinstance(Pot,list) \
+            and nu.all([isinstance(p,planarPotential) for p in Pot]):
         sum= 0.
         for pot in Pot:
             if nonAxi:
@@ -571,8 +652,8 @@ def evaluateplanarPotentials(R,Pot,phi=None,t=0.,dR=0,dphi=0):
             return Pot(R,phi=phi,t=t,dR=dR,dphi=dphi)
         else:
             return Pot(R,t=t,dR=dR,dphi=dphi)
-    else:
-        raise TypeError("Input to 'evaluateplanarPotentials' is neither a Potential-instance or a list of such instances")
+    else: #pragma: no cover 
+        raise PotentialError("Input to 'evaluatePotentials' is neither a Potential-instance or a list of such instances")
 
 def evaluateplanarRforces(R,Pot,phi=None,t=0.):
     """
@@ -611,7 +692,8 @@ def evaluateplanarRforces(R,Pot,phi=None,t=0.):
         nonAxi= Pot.isNonAxi
     if nonAxi and phi is None:
         raise PotentialError("The (list of) planarPotential instances is non-axisymmetric, but you did not provide phi")
-    if isinstance(Pot,list):
+    if isinstance(Pot,list) \
+            and nu.all([isinstance(p,planarPotential) for p in Pot]):
         sum= 0.
         for pot in Pot:
             if nonAxi:
@@ -624,8 +706,8 @@ def evaluateplanarRforces(R,Pot,phi=None,t=0.):
             return Pot.Rforce(R,phi=phi,t=t)
         else:
             return Pot.Rforce(R,t=t)
-    else:
-        raise TypeError("Input to 'evaluateplanarRforces' is neither a Potential-instance or a list of such instances")
+    else: #pragma: no cover 
+        raise PotentialError("Input to 'evaluatePotentials' is neither a Potential-instance or a list of such instances")
 
 def evaluateplanarphiforces(R,Pot,phi=None,t=0.):
     """
@@ -664,7 +746,8 @@ def evaluateplanarphiforces(R,Pot,phi=None,t=0.):
         nonAxi= Pot.isNonAxi
     if nonAxi and phi is None:
         raise PotentialError("The (list of) planarPotential instances is non-axisymmetric, but you did not provide phi")
-    if isinstance(Pot,list):
+    if isinstance(Pot,list) \
+            and nu.all([isinstance(p,planarPotential) for p in Pot]):
         sum= 0.
         for pot in Pot:
             if nonAxi:
@@ -677,8 +760,8 @@ def evaluateplanarphiforces(R,Pot,phi=None,t=0.):
             return Pot.phiforce(R,phi=phi,t=t)
         else:
             return Pot.phiforce(R,t=t)
-    else:
-        raise TypeError("Input to 'evaluateplanarphiforces' is neither a Potential-instance or a list of such instances")
+    else: #pragma: no cover 
+        raise PotentialError("Input to 'evaluatePotentials' is neither a Potential-instance or a list of such instances")
 
 def evaluateplanarR2derivs(R,Pot,phi=None,t=0.):
     """
@@ -717,7 +800,8 @@ def evaluateplanarR2derivs(R,Pot,phi=None,t=0.):
         nonAxi= Pot.isNonAxi
     if nonAxi and phi is None:
         raise PotentialError("The (list of) planarPotential instances is non-axisymmetric, but you did not provide phi")
-    if isinstance(Pot,list):
+    if isinstance(Pot,list) \
+            and nu.all([isinstance(p,planarPotential) for p in Pot]):
         sum= 0.
         for pot in Pot:
             if nonAxi:
@@ -730,8 +814,61 @@ def evaluateplanarR2derivs(R,Pot,phi=None,t=0.):
             return Pot.R2deriv(R,phi=phi,t=t)
         else:
             return Pot.R2deriv(R,t=t)
-    else:
-        raise TypeError("Input to 'evaluateplanarR2derivs' is neither a Potential-instance or a list of such instances")
+    else: #pragma: no cover 
+        raise PotentialError("Input to 'evaluatePotentials' is neither a Potential-instance or a list of such instances")
+
+def LinShuReductionFactor(axiPot,R,sigmar,nonaxiPot=None,
+                          k=None,m=None,OmegaP=None):
+    """
+    NAME:
+
+       LinShuReductionFactor
+
+    PURPOSE:
+
+       Calculate the Lin & Shu (1966) reduction factor: the reduced linear response of a kinematically-warm stellar disk to a perturbation
+
+    INPUT:
+
+       axiPot - The background, axisymmetric potential
+
+       R - Cylindrical radius
+       
+       sigmar - radial velocity dispersion of the population
+
+       Then either provide:
+
+       1) m= m in the perturbation's m x phi (number of arms for a spiral)
+
+          k= wavenumber (see Binney & Tremaine 2008)
+
+          OmegaP= pattern speed
+
+       2) nonaxiPot= a non-axisymmetric Potential instance (such as SteadyLogSpiralPotential) that has functions that return OmegaP, m, and wavenumber
+
+    OUTPUT:
+
+       reduction factor
+
+    HISTORY:
+
+       2014-08-23 - Written - Bovy (IAS)
+
+    """
+    from galpy.potential import omegac, epifreq
+    if nonaxiPot is None and (OmegaP is None or k is None or m is None):
+        raise IOError("Need to specify either nonaxiPot= or m=, k=, OmegaP= for LinShuReductionFactor")
+    elif not nonaxiPot is None:
+        OmegaP= nonaxiPot.OmegaP()
+        k= nonaxiPot.wavenumber(R)
+        m= nonaxiPot.m()
+    tepif= epifreq(axiPot,R)
+    s= m*(OmegaP-omegac(axiPot,R))/tepif
+    chi= sigmar**2.*k**2./tepif**2.
+    return (1.-s**2.)/nu.sin(nu.pi*s)\
+        *integrate.quad(lambda t: nu.exp(-chi*(1.+nu.cos(t)))\
+                            *nu.sin(s*t)*nu.sin(t),
+                        0.,nu.pi)[0]
 
 def plotplanarPotentials(Pot,*args,**kwargs):
     """
@@ -785,12 +922,12 @@ def plotplanarPotentials(Pot,*args,**kwargs):
         grid= kwargs['grid']
         kwargs.pop('grid')
     else:
-        grid= 1000 #avoid zero
+        grid= 100 #avoid zero
     if kwargs.has_key('gridx'):
         gridx= kwargs['gridx']
         kwargs.pop('gridx')
     else:
-        gridx= 1000 #avoid zero
+        gridx= 100 #avoid zero
     if kwargs.has_key('gridy'):
         gridy= kwargs['gridy']
         kwargs.pop('gridy')
@@ -803,17 +940,16 @@ def plotplanarPotentials(Pot,*args,**kwargs):
         savefilename= None
     isList= isinstance(Pot,list)
     nonAxi= ((isList and Pot[0].isNonAxi) or (not isList and Pot.isNonAxi))
-    if not savefilename == None and os.path.exists(savefilename):
+    if not savefilename is None and os.path.exists(savefilename):
         print "Restoring savefile "+savefilename+" ..."
         savefile= open(savefilename,'rb')
         potR= pickle.load(savefile)
-        Rs= pickle.load(savefile)
         if nonAxi:
             xs= pickle.load(savefile)
             ys= pickle.load(savefile)
         else:
             Rs= pickle.load(savefile)
-            savefile.close()
+        savefile.close()
     else:
         if nonAxi:
             xs= nu.linspace(xrange[0],xrange[1],gridx)
@@ -833,8 +969,8 @@ def plotplanarPotentials(Pot,*args,**kwargs):
             potR= nu.zeros(grid)
             for ii in range(grid):
                 potR[ii]= evaluateplanarPotentials(Rs[ii],Pot)
-        if not savefilename == None:
-            print "Writing savefile "+savefilename+" ..."
+        if not savefilename is None:
+            print "Writing planar savefile "+savefilename+" ..."
             savefile= open(savefilename,'wb')
             pickle.dump(potR,savefile)
             if nonAxi:
@@ -842,7 +978,7 @@ def plotplanarPotentials(Pot,*args,**kwargs):
                 pickle.dump(ys,savefile)
             else:
                 pickle.dump(Rs,savefile)
-                savefile.close()
+            savefile.close()
     if nonAxi:
         if not kwargs.has_key('origin'):
             kwargs['origin']= 'lower'
@@ -870,7 +1006,7 @@ def plotplanarPotentials(Pot,*args,**kwargs):
                                 yrange=yrange,**kwargs)
     else:
         kwargs['xlabel']=r"$R/R_0$"
-        kwargs['ylabel']=r"$\Phi(R)$",
+        kwargs['ylabel']=r"$\Phi(R)$"
         kwargs['xrange']=Rrange
         return plot.bovy_plot(Rs,potR,*args,**kwargs)
                               
