@@ -108,13 +108,13 @@ void bovy_rk4(void (*func)(double t, double *q, double *a,
   //We're done
 }
 
-inline void bovy_rk4_onestep(void (*func)(double t, double *q, double *a,
-					  int nargs, struct potentialArg * potentialArgs),
-			     int dim,
-			     double * yn,double * yn1,
-			     double tn, double dt,
-			     int nargs, struct potentialArg * potentialArgs,
-			     double * ynk, double * a){
+void bovy_rk4_onestep(void (*func)(double t, double *q, double *a,
+				   int nargs, struct potentialArg * potentialArgs),
+		      int dim,
+		      double * yn,double * yn1,
+		      double tn, double dt,
+		      int nargs, struct potentialArg * potentialArgs,
+		      double * ynk, double * a){
   int ii;
   //calculate k1
   func(tn,yn,a,nargs,potentialArgs);
@@ -210,16 +210,16 @@ void bovy_rk6(void (*func)(double t, double *q, double *a,
   k_7 = step * dxdt(start + step, x[i] + (9*k_1 - 36*k_2 + 63*k_3 +
 72*k_4 -64*k_6)/44)
 */
-inline void bovy_rk6_onestep(void (*func)(double t, double *q, double *a,
-					  int nargs, struct potentialArg * potentialArgs),
-			     int dim,
-			     double * yn,double * yn1,
-			     double tn, double dt,
-			     int nargs, struct potentialArg * potentialArgs,
-			     double * ynk, double * a,
-			     double * k1, double * k2,
-			     double * k3, double * k4,
-			     double * k5){
+void bovy_rk6_onestep(void (*func)(double t, double *q, double *a,
+				   int nargs, struct potentialArg * potentialArgs),
+		      int dim,
+		      double * yn,double * yn1,
+		      double tn, double dt,
+		      int nargs, struct potentialArg * potentialArgs,
+		      double * ynk, double * a,
+		      double * k1, double * k2,
+		      double * k3, double * k4,
+		      double * k5){
   int ii;
   //calculate k1
   func(tn,yn,a,nargs,potentialArgs);
@@ -268,10 +268,6 @@ inline void bovy_rk6_onestep(void (*func)(double t, double *q, double *a,
   //yn1 is new value
 }
 
-inline void save_rk(int dim, double *yo, double *result){
-  int ii;
-  for (ii=0; ii < dim; ii++) *result++= *yo++;
-}
 double rk4_estimate_step(void (*func)(double t, double *y, double *a,int nargs, struct potentialArg *),
 			 int dim, double *yo,
 			 double dt, double *t,
@@ -565,11 +561,11 @@ double bovy_dopr54_actualstep(void (*func)(double t, double *y, double *a,int na
   static const double b5= -2187./6784.;
   static const double b6= 11./84.;
   //coefficients of the error estimate
-  static const double be1= b1-5179./57600.;
-  static const double be3= b3-7571./16695.;
-  static const double be4= b4-393./640.;
-  static const double be5= b5+92097./339200.;
-  static const double be6= b6-187./2100.;
+  const double be1= b1-5179./57600.;
+  const double be3= b3-7571./16695.;
+  const double be4= b4-393./640.;
+  const double be5= b5+92097./339200.;
+  const double be6= b6-187./2100.;
   static const double be7= -1./40.;
   int ii;
   //setup yn1
